# command.py
#
# Copyright (C) 2014 Tim Marston <tim@edm.am>
#
# This file is part of stdhome (hereafter referred to as "this program").
# See http://ed.am/dev/stdhome for more information.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


import os
import stdhome.the as the
from stdhome.walker.walker import Walker


class Command:
	"""Base class for command classes.
	"""

	@staticmethod
	def resolve_homedir_file( file ):
		"""Given a filename, which could be absolute or relative to the CWD, this
		returns a pair of 'resolved' filenames: the name of the file relative to
		the homedir and the full, absolute filename.  Neither of the returned
		filenames are guaranteed nor required to exist.  But the supplied
		filename will cause an error if it is not iteself or when resolved (if
		it is a symlink) under the homedirectory.

		"""
		home_dir_prefix = os.path.realpath( the.full_home_dir ) + os.sep

		# obtain absolute filename
		abs_file = os.path.abspath( file )

		# if absolute filename is not under home directory, attempt to
		# resolve symlinks
		if abs_file[ : len( home_dir_prefix ) ] != home_dir_prefix:
			parts = os.path.split( file )
			if os.path.exists( parts[ 0 ] ):
				abs_file = os.path.join(
					os.path.realpath( parts[ 0 ] ), parts[ 1 ] )

		# absolute file must now be under home directory and exist
		if abs_file[ : len( home_dir_prefix ) ] != home_dir_prefix:
			raise the.program.FatalError(
					'not under home directory: %s' % file )

		# relative file
		rel_file = abs_file[ len( home_dir_prefix ) : ]

		return ( rel_file, abs_file )


	@staticmethod
	def expand_files( files, recurse = True ):
		"""Returns a unique, sorted list of relative files, calculated from the list
		provided, which is made up from individual files and directories
		relative to the CWD (and which must be contained within the home
		directory, although the files need not actually exist in the home
		directory).  All files must exist in the repository.  Directories are
		recursed in to as required.

		"""

		ret = set()

		# iterate through file list
		for file in files:
			( rel_file, abs_file ) = self.resolve_homedir_file( file )

			# check that file exists in repository
			repo_file = os.path.join( the.repo.full_dir, rel_file )
			if not os.path.lexists( repo_file ):
				raise the.program.FatalError(
					'not managed by stdhome: %s' % rel_file )

			# append the file or directory tree
			ret.update( Walker.generate_walk_list(
				the.repo.full_dir, rel_file, recurse ) )

		return sorted( set ( ret ) )


	@staticmethod
	def print_stage_commands_notice():
		#      01234567890123456789012345678901234567890123456789012345678901234567890123456789
		print("In addition to using the primary commands (such as add and remove) to modify a")
		print("remote repository, you can also set up changes in the local repository and")
		print("manually commit them in one go to a remote repository when you're ready.  This")
		print("is done with the staging commands.  Note, some primary commands will not work")
		print("when there are staged changes.  You can revert all staged changes with the")
		print("stage-revert command.")
		
